/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _flexSerial_h_
#define _flexSerial_h_

#ifdef __CYGWIN__
#include <windows.h>
#else
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#endif

/** 
* @file flexSerial.h
* @brief Serial communication
* @author Nastasi Christian
* @date 2007-12-17
*/
 
/** 
* @brief The serial object data type
*/
typedef struct {
	#ifdef __CYGWIN__
	HANDLE handlerCom;
	DCB ctrl;
	BOOL bPortReady;
	COMMTIMEOUTS CommTimeouts;
	#else
	int fd;
	#endif
} Flex_Serial;


/** 
* @brief Open the serial port
* 
* @param com The Serial Object to open
* @param port_name A valid serial port name
* @param baud_rate A valid serial baud rate
* 
* @return \c 1 if no error occurs, \c 0 otherwise.
*/
int flex_Serial_init(Flex_Serial* com, const char* port_name, unsigned int baud_rate);

/** 
* @brief Close the serial port
* 
* @param com The Serial Object to close
*/
void flex_Serial_close(Flex_Serial* com);

/** 
* @brief Read one byte from the serial
* 
* @param com The serial object
* @param data The destination byte
* 
* @return \c 1 if no error occurs, a negative value otherwise
*/
int flex_Serial_getc(Flex_Serial* com, char *data);

/** 
* @brief Write one byte to the serial
* 
* @param com The serial object
* @param data The byte to write
* 
* @return \c 1 if no error occurs, a negative value otherwise
*/
int flex_Serial_putc(Flex_Serial* com, char data);

/** 
* @brief Read \c data_size bytes from the serial
* 
* @param com The serial object
* @param data The destination buffer capable to store the \c data_size bytes
* @param data_size The number of byte to read
* 
* @return the number of bytes read from serial if no error occurs, a negative value otherwise
*/
int flex_Serial_read(Flex_Serial* com, char *data, unsigned int data_size);

/** 
* @brief Write \c data_size bytes to the serial
* 
* @param com The serial object
* @param data The source buffer with the \c data_size bytes
* @param data_size The number of byte to write
* 
* @return the number of bytes written to serial if no error occurs, a negative value otherwise
*/
int flex_Serial_write(Flex_Serial* com, char* data, unsigned int data_size);


#endif
