#include "flexGUI.h"
#include "flexSerial.h"
#include "flexCAM.h"
#include <iostream>
#include <signal.h>

using namespace std;

#ifndef __CYGWIN__
#include <unistd.h> // TO REMOVE
#include <stdlib.h> //TO REMOVE
#include <string.h> //TO REMOVE
#define flex_sleep(sec) sleep((sec))
#else
#define flex_sleep(sec) Sleep((sec)*1000) //Windows API
#endif

#define PROG_NAME "flex_cam_serial"
#define PROG_TAG "\nFlex-CAM Image Serial Reader :  "
#define MIN_ARGS 2

/***************************************************************/

void print_help();

void close_handler(int signo); 

FlexCAM driver;

/***************************************************************/

int main(int argc, char *argv[])
{
	const char* port_name=NULL;
	unsigned char type=FLEX_CAM_JPEG;
	unsigned char size=FLEX_CAM_SMALL;
	unsigned long int baud_rate=38400;
	unsigned char* image;
	unsigned int w,h,c;
	bool is_video;
	bool first_gui=true;
	unsigned long int frame_count=0;
	const char * __rgb="Color RGB";
	const char * __jpeg="Color JPEG";
	const char * __gray="Gray scale";
	const char * out_str;

	struct sigaction n,v;

	/* Args parsing ... */
	argc--;
	if(argc < MIN_ARGS){
		print_help();
		return(EXIT_SUCCESS);
	}


	if(strcmp(argv[1],"snap")==0) is_video=false;
	else if(strcmp(argv[1],"video")==0) is_video=true;
	else{
		cout << PROG_TAG << "Error: Bad Params, check out help "<< endl;
		print_help();
		return(EXIT_FAILURE);
	}

	if(strcmp(argv[2],"jpeg")==0) type=FLEX_CAM_JPEG;
	else if(strcmp(argv[2],"rgb16")==0) type=FLEX_CAM_RGB16;
	else if(strcmp(argv[2],"rgb12")==0) type=FLEX_CAM_RGB12;
	else if(strcmp(argv[2],"gray2")==0) type=FLEX_CAM_GRAY2;
	else if(strcmp(argv[2],"gray4")==0) type=FLEX_CAM_GRAY4;
	else if(strcmp(argv[2],"gray8")==0) type=FLEX_CAM_GRAY8;
	else{
		cout << PROG_TAG << "Error: Bad Params, check out help "<< endl;
		print_help();
		return(EXIT_FAILURE);
	}

	/*
	if(strcmp(argv[3],"small")==0) size=FLEX_CAM_SMALL; 
	else if(strcmp(argv[3],"med")==0) size=FLEX_CAM_MED; 
	else if(strcmp(argv[3],"large")==0) size=FLEX_CAM_LARGE;
	else{
		cout << PROG_TAG << "Error: Bad Params, check out help "<< endl;
		print_help();
		return(EXIT_FAILURE);
	}
	*/

	if(argc > MIN_ARGS){
		port_name=argv[3];
		// TODO: check the port_name!
	}
	/*
	if(argc > MIN_ARGS+1){
		if(strcmp(argv[4],"9600")==0) baud_rate=9600;
		else if(strcmp(argv[4],"19200")==0) baud_rate=19200;
		else if(strcmp(argv[4],"38400")==0) baud_rate=38400;
		else if(strcmp(argv[4],"57600")==0) baud_rate=57600;
		else if(strcmp(argv[4],"115200")==0) baud_rate=115200;
		else{
			cout << PROG_TAG << "Error: Bad Params, check out help "<< endl;
			print_help();
			return(EXIT_FAILURE);
		}
	}
	*/

  //  if(argc > MIN_ARGS){
//		if(strcmp(argv[3],"9600")==0) baud_rate=9600;
//		else if(strcmp(argv[3],"19200")==0) baud_rate=19200;
//		else if(strcmp(argv[3],"38400")==0) baud_rate=38400;
//		else if(strcmp(argv[3],"57600")==0) baud_rate=57600;
//		else if(strcmp(argv[3],"115200")==0) baud_rate=115200;
//		else{
//			cout << PROG_TAG << "Error: Bad Params, check out help "<< endl;
//			print_help();
//			return(EXIT_FAILURE);
//		}
//	}

	//Driver (Class) configuration and settings:
	n.sa_handler = close_handler;
	n.sa_flags=0;
	sigemptyset(&(n.sa_mask));

	if(sigaction(SIGINT, &n,&v)==-1){
		cout << PROG_TAG << "Handler Install Error " << endl;
		return EXIT_FAILURE;
	}

	if(!driver.set_image_type(type, size, is_video)){
		cout << PROG_TAG << "Set Image Type Error : ErrorCode=" << driver.get_error() << endl;
		return EXIT_FAILURE;
	} 

	driver.set_baud_rate_pc(baud_rate);

	w = driver.get_width();
	h = driver.get_height();
	c = driver.get_channel();

	FlexGUI gui(w+40,h+30,"Flex-CAM");

	if(type<=FLEX_CAM_GRAY8)
		out_str=__gray;
	else if(type<=FLEX_CAM_RGB16)
		out_str=__rgb;
	else
		out_str=__jpeg;

	cout << PROG_TAG << "Configuration: " << endl << endl 
	     << "\twidth  = " << w << endl 
	     << "\theight = " << h << endl 
	     << "\ttype   = " << out_str << endl
	     << "\tbaud rate PC-Flex = " << baud_rate << endl;

	cout << PROG_TAG << "Press Enter to Start ";
	getchar();
	
	//Connection:
	cout << PROG_TAG << "Connecting to port ... ";
	fflush(stdout);
	flex_sleep(1);
	if(!driver.connect(port_name)){
		cout << PROG_TAG << "Connection Error : ErrorCode=" << driver.get_error() << endl;
		return EXIT_FAILURE;
	}
	cout << "Done!";
	
	//Configuration:
	cout << PROG_TAG << "Configuring Flex: ";
	fflush(stdout);
	flex_sleep(2);
	if(!driver.configure(port_name)){
		cout << PROG_TAG << "Configuration Error : ErrorCode=" << driver.get_error() << endl;
		return EXIT_FAILURE;
	}
	cout << "Done!";
	flex_sleep(2);
	
	//Cycle:
	while(1)
	{
		cout << PROG_TAG << "Get image : frame n. " << ++frame_count <<" ... ";
		fflush(stdout);

		if(!driver.get_image(image)){
			cout << PROG_TAG << "Get Image Error : ErrorCode=" << driver.get_error() << endl;
			return EXIT_FAILURE;
		}
		cout << "Done!";

		if(first_gui){
			first_gui=false;
			gui.show_gui();
			Fl::check();
		}

		if(!gui.image_show(image, w, h, c)){
			cout << PROG_TAG << "Show Image Error " << endl;
			return EXIT_FAILURE;
		}

		Fl::check();

		if(!is_video){
			cout << endl << PROG_TAG << "Press Enter for another snap ";
			getchar();
		}

	
	}// END of While
	
	cout << PROG_TAG << "Closing connection ... ";
	driver.close();
	cout << "Done!";
	

	cout << PROG_TAG << "Exiting ... Bye! " << endl;

	return EXIT_SUCCESS;
}

/***************************************************************/

void close_handler(int signo)
{
	cout << PROG_TAG << "Closing connection ... ";
	driver.close();
	cout << "Done!";
	
	cout << PROG_TAG << "Exiting ... Bye! " << endl;

	exit(EXIT_SUCCESS);
} 

/***************************************************************/

void print_help()
{	
	printf("\n");
	printf("*** Flex-CAM Image Serial Reader ***"); printf("\n\n");

	//printf("Usage :  "); printf(PROG_NAME); printf(" <video|snap> <type> [port [baud]]\n\n");
	printf("Usage :  "); printf(PROG_NAME); printf(" <video|snap> <type> [port]\n\n");

	printf("  video : Configure flex for video (continous image stream) \n"); 
	printf("  snap  : Configure flex for snapshot (single image) \n\n"); 

	printf("  type  : the type of the image, could be one of the following: \n"); 
	printf("            gray2 - Gray scale 2 bit \n"); 
	printf("            gray4 - Gray scale 4 bit \n"); 
	printf("            gray8 - Gray scale 8 bit \n"); 
	printf("            rgb16 - RGB 16 bit (R:5 - G:6 - B:5)\n"); 
	printf("            rgb12 - RGB 12 bit (R:4 - G:4 - B:4)\n"); 
	printf("            jpeg  - Jpeg compressed image \n\n"); 

	//printf("  port  : the serial port to connect; default is com1 (windows) or /dev/ttys0 (unix) \n\n"); 
	
//	printf("  baud  : the PC-Flex serial baud rate; available values are:\n"); 
//	printf("            9600, 19200, 38400(default), 57600, 115200 \n\n\n"); 
    //printf("            9600, 19200, 38400(default) \n\n\n"); 

	printf("  port  : the serial port to connect; default is com1 (windows)\n\n"); 

	printf("Examples : "); printf(PROG_NAME); printf(" video jpeg\n");
	printf("           "); printf(PROG_NAME); printf(" snap  rgb12\n");
	printf("           "); printf(PROG_NAME); printf(" video gray2 com1\n");
	    printf("           "); printf(PROG_NAME); printf(" video jpeg com3\n\n");

	printf("NOTE: Use Ctrl-c to interrupt the execution\n\n");

	printf("Version : "); printf("0.1\n");
	printf("Authors : "); printf("Flex side - Di Stefano Dario (Engineer, University of Pisa)\n");
	printf("          "); printf("PC side   - Nastasi Christian (Engineer, University of Pisa)\n\n");

	printf("\n");

	exit(1);
}

/***************************************************************/



